/*****************************************************************************/
/*                                                                           */
/* FILENAME                                                                  */
/*   sampling_bits.c                                                         */
/*                                                                           */
/* DESCRIPTION                                                               */
/*   Selecting the number of bits resolution of the ADC / DAC on the         */
/*   TMS320C5416 DSK.                                                        */
/*                                                                           */
/*   Takes an input from a microphone or from a CD player and feeds straight */
/*   through to loudspeakers. The user switches select the number of bits of */
/*   resolution of the ADC/DAC.                                              */
/*                                                                           */
/* REVISION                                                                  */
/*   Revision: 1.00	                                                         */
/*   Author  : Richard Sikora                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* HISTORY                                                                   */
/*   Revision 1.00                                                           */
/*   1st November 2002. Created by Richard Sikora from TMS320C5402 code.     */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>    /* Required for functions printf() and puts()          */

/*****************************************************************************/
/*                                                                           */
/* The following line takes the name of the module, here sampling_bits.c and */
/* adds the letters cfg.h                                                    */
/*                                                                           */
/*****************************************************************************/

#include "sampling_bitscfg.h"

#include "dsk5416.h"
#include "dsk5416_pcm3002.h"

#include "bargraph.h"
#include "switches.h"


/*****************************************************************************/
/* Configuration setup of registers of PCM3002 Codec                         */
/*****************************************************************************/

DSK5416_PCM3002_Config setup = {

    0x1FF,      // Set-Up Reg 0 - Left channel DAC attenuation
    0x1FF,      // Set-Up Reg 1 - Right channel DAC attenuation
    0x0,        // Set-Up Reg 2 - Various ctl e.g. power-down modes
    0x0         // Set-Up Reg 3 - Codec data format control
};

/*****************************************************************************/
/*                                                                           */
/* For compatibility with pcm3002 read / write, these variables must         */
/* be declared as Int16 or short int, rather than int.                       */
/*                                                                           */
/*****************************************************************************/

Int16 left_input;
Int16 left_output; 
Int16 right_input;
Int16 right_output; 

/*****************************************************************************/
/* UserTask()                                                                */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* The main user task.                                                       */
/*                                                                           */
/* Note that this task is not called by main(). It is scheduled by DSP/BIOS  */
/*                                                                           */
/*****************************************************************************/
 
void UserTask()
{
    DSK5416_PCM3002_CodecHandle hCodec;
    long i;
    unsigned int switch_value; 
        
    /* Start the codec */
    hCodec = DSK5416_PCM3002_openCodec(0, &setup);
 
    /* Display project details on StdOut. \n is important. */
 
    puts("TMS320C5416 DSK: Changing the number of bits per sample.\n");
          
    for (i = 0; i < 12000000; i++)
    {
      /* Read left input channel */    
      while (!DSK5416_PCM3002_read16(hCodec, &left_input));

      /* Write to left output channel */      
      while (!DSK5416_PCM3002_write16(hCodec, left_output));
      
      /* Read right input channel */
      while (!DSK5416_PCM3002_read16(hCodec, &right_input));
      
      /* Write to right output channel */
      while (!DSK5416_PCM3002_write16(hCodec, right_output));
      
      /* Read user switches on DSK and display their meaning on Stdout. */
      switch_value = switch_status_display();
  
      if ( 0 == switch_value)
       {
         /* 16-bit resolution */
         left_output = left_input;  
         right_output = right_input;
       }
      else if ( 1 == switch_value)
       {
         /* 15-bit resolution */
         left_output = ( left_input & 0xFFFE );  
         right_output = ( right_input & 0xFFFE );         
       }  
      else if ( 2 == switch_value)
       {
         /* 14-bit resolution */
         left_output = ( left_input & 0xFFFC );  
         right_output = ( right_input & 0xFFFC );           
       } 
      else if ( 3 == switch_value)
       {
         /* 13-bit resolution */
         left_output = ( left_input & 0xFFF8 );  
         right_output = ( right_input & 0xFFF8 );          
       }       
      else if ( 4 == switch_value)
       {
         /* 12-bit resolution */
         left_output = ( left_input & 0xFFF0 );  
         right_output = ( right_input & 0xFFF0 );        
       }         
      else if ( 5 == switch_value)
       {
         /* 11-bit resolution */
         left_output = ( left_input & 0xFFE0 );  
         right_output = ( right_input & 0xFFE0 );          
       }   
      else if ( 6 == switch_value)
       {
         /* 10-bit resolution */
         left_output = ( left_input & 0xFFC0 );  
         right_output = ( right_input & 0xFFC0 );  
       }       
      else if ( 7 == switch_value)
       {
         /* 9-bit resolution */
         left_output = ( left_input & 0xFF80 );  
         right_output = ( right_input & 0xFF80 );          
       }        
      else if ( 8 == switch_value)
       {
         /* 8-bit resolution */
         left_output = ( left_input & 0xFF00 );  
         right_output = ( right_input & 0xFF00 );           
       }        
      else if ( 9 == switch_value )
       {
         /* 7-bit resolution */
         left_output = ( left_input & 0xFE00 );  
         right_output = ( right_input & 0xFE00 );           
       } 
      else if ( 10 == switch_value)
       {
         /* 6-bit resolution */
         left_output = ( left_input & 0xFC00 );  
         right_output = ( right_input & 0xFC00 );           
       } 
      else if ( 11 == switch_value)
       {
         /* 5-bit resolution */
         left_output = ( left_input & 0xF800 );  
         right_output = ( right_input & 0xF800 );         
       }
      else if ( 12 == switch_value)
       {
          /* Not used */
       }        
      else if ( 13 == switch_value)
       {
          /* Not used */
       }
      else if ( 14 == switch_value)
       {
          /* Not used */
       }        
      else if ( 15 == switch_value)
       {
          /* Not used */
       }               
  
      /* Display greater of two outputs on bargraph */
      bargraph_6dB ( left_output, right_output);
     }

    /* Finished processing. Close the codec */
    DSK5416_PCM3002_closeCodec(hCodec);
    
    puts("TMS320C5416 DSK program has terminated\n");
}

/*****************************************************************************/
/* main()                                                                    */
/*****************************************************************************/

void main()
{
    /* Initialize the board support library                         */
    /* There is no need to initialize the DIP switches and the LEDs */
    
    DSK5416_init();
    
    /* All other functions are scheduled by DSP/BIOS */
}

/******************************************************************************/
/* End of sampling_bits.c                                                     */
/******************************************************************************/

